use tokio::io::AsyncReadExt;

pub struct TravelPathInfo{
    pub duration: chrono::Duration,
    pub polyline: String
}
pub async fn calc_travel_path(path:&[crate::types::Coords])->anyhow::Result<TravelPathInfo>{
    let mut google_api_key = String::new(); 
    tokio::fs::File::open("GOOGLE_API_KEY").await?.read_to_string(&mut google_api_key).await?;
    let client = reqwest::Client::new();
    let response = client.post("https://routes.googleapis.com/directions/v2:computeRoutes")
        .body(serde_json::to_string(&build_route_body_pain(path))?)
        .header("Content-Type","application/json")
        .header("X-Goog-Api-Key:", google_api_key)
        .header("X-Goog-FieldMask", "routes.duration,routes.polyline.encodedPolyline,routes.legs")
        .send().await?;
    let response = serde_json::from_str::<RouteResponse>(response.text().await?.as_str())?;

    Ok(response.into())
}
impl From<RouteResponse> for TravelPathInfo{
    fn from(value: RouteResponse) -> Self {
        let RouteResponse { mut routes } = value;
        let head = routes.pop().unwrap();
        let duration = chrono::Duration::seconds(head.duration[0..head.duration.len()-1].parse().expect("Expected google's api to return [0-9]s"));
        Self{
            polyline: head.polyline.encodedPolyline,
            duration
        }
    }
}
#[derive(serde::Deserialize,serde::Serialize)]
struct RouteResponse{
    routes:Vec<Route>
}
#[derive(serde::Deserialize,serde::Serialize)]
struct Route{
    duration: String,
    polyline: PolyLine
}
#[derive(serde::Deserialize,serde::Serialize)]
struct PolyLine{
    #[allow(non_snake_case)]
    encodedPolyline: String
}
#[derive(serde::Deserialize,serde::Serialize)]
struct Location{
    #[allow(non_snake_case)]
    latLng:LocIn
}
#[derive(serde::Deserialize,serde::Serialize)]
struct LocIn{
    latitude: f64,
    longitude: f64
}
fn build_route_body_pain(path:&[crate::types::Coords])->serde_json::Value{
    let time = chrono::offset::Utc::now()
        .to_rfc3339_opts(chrono::SecondsFormat::Nanos,true);
    let inter = &path[1..path.len()-1];
    let mut val = serde_json::json!({
      "origin":{
        "location": path[0],
        "sideOfRoad": true
      },
      "destination":{
        "location":path[path.len()-1]
      },
      "intermediates": inter,
      "travelMode": "DRIVE",
      "routingPreference": "TRAFFIC_UNAWARE",
      "departureTime": time,
      "computeAlternativeRoutes": false,
      "routeModifiers": {
        "avoidTolls": false,
        "avoidHighways": false,
        "avoidFerries": false
      },
      "languageCode": "en-US",
      "units": "IMPERIAL"
    });
    val
}
