extern crate tokio;
extern crate reqwest;
extern crate tide;
extern crate anyhow;
extern crate chrono;
//extern crate sea_orm;

mod state;
mod route;
mod stop;
mod google;
mod types;
mod path_calc;

use tide::prelude::*;

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    let mut app = tide::with_state(state::State::new().await?);

    app.at("/path").get(transit_path);
    app.at("/time_to_arrive").get(transit_estimate);
    app.at("/route_draw").get(bus_route_draw);

    // TODO: change to 0.0.0.0 when docker image done
    app.listen("127.0.0.1:8080").await?;
    Ok(())
}


/// return the best path of buses with the request specifying ISO time, lat and longitude in query params
/// Example return
/// {
///     "path": [
///         {
///             "route_name": "WS Outbound",
///             "enter_stop_id": 3,
///             "exit_stop_id": 10,
///             "enter_stop_coords": {"lat":23.3512, "lon": 1231.123123}
///         },
///         ...
///     ]
/// }
async fn transit_path(mut req: tide::Request<state::State>)->tide::Result{


    Ok("".into())
}

/// return the num of estimated seconds for stop ids given in query params, also need to specify
/// service and route id
/// example return:
/// 23
async fn transit_estimate(mut req: tide::Request<state::State>)->tide::Result{
    let query = req.query::<TransitEstimateQuery>()?;
    let route = match query.service.as_str(){
        "OCCT"=>{
            let routes = route::occt_routes_get().await?;
            let Some(route) = routes.into_iter().find(|route| route.id == query.route) else {return Ok(tide::Response::builder(400).body("Error: invalid id").into())};
            route
        },
        "BC"=>{
            let routes = route::broome_county_routes_get().await?;
            let Some(route) = routes.into_iter().find(|route|route.id == query.route) else {return Ok(tide::Response::builder(400).body("Error: invalid id").into())};
            route
        },
        _ => {
            return Ok(tide::Response::builder(400).body("Error: invalid service").into())
        }
    };
    let Some((from_idx,_)) = route.stops.iter().enumerate().find(|(_,stop)|stop.id == query.from) 
        else {return Ok(tide::Response::builder(400).body("Error: invalid id").into())};
    let Some((to_idx,_)) = route.stops.iter().enumerate().find(|(_,stop)|stop.id == query.to) 
        else {return Ok(tide::Response::builder(400).body("Error: invalid id").into())};

    Ok(format!("{}", path_calc::bus_travel_time(route, from_idx, to_idx).await?).into())
}
#[derive(serde::Deserialize)]
struct TransitEstimateQuery{
    service:String,
    from:u32,
    to:u32,
    route: u32
}
/// return the coord pairs as json and line color forgiven route id in query params
/// Example return:
/// {"color":"#0000ff", "stops":[{"lat":-75.5,"lon":23},...]}
async fn bus_route_draw(mut req: tide::Request<state::State>)->tide::Result{
    let query = req.query::<RouteDrawQuery>()?;
    let route = match query.service.as_str() {
        "OCCT" => {
            let routes = route::occt_routes_get().await?;
            let Some(route) = routes.into_iter().find(|route| route.id == query.id) else {return Ok(tide::Response::builder(400).body("Error: invalid id").into())};
            route
        },
        "BC" => {
            let routes = route::broome_county_routes_get().await?;
            let Some(route) = routes.into_iter().find(|route|route.id == query.id) else {return Ok(tide::Response::builder(400).body("Error: invalid id").into())};
            route
        },
        _ => {
            return Ok(tide::Response::builder(400).body("Error: invalid service").into())
        }
        
    };
    let ret = stop::RouteDraw {
        color: route.color,
        stops: route.stops.into_iter().map(|stop|{types::Coords{lat:stop.lat,lon:stop.lon}}).collect()
    };
    Ok(serde_json::to_string(&ret)?.into())
}

#[derive(serde::Deserialize)]
struct RouteDrawQuery{
    id: u32,
    service: String
}

