pub async fn bus_travel_time(route:crate::route::Route, from_idx:usize, to_idx:usize)->anyhow::Result<chrono::Duration>{
    let coords:Vec<crate::types::Coords> = route.stops[from_idx..=to_idx].iter()
        .map(|stop|crate::types::Coords{lat:stop.lat, lon:stop.lon})
        .collect();
    let travel_time = crate::google::calc_travel_path(&coords).await?;
    Ok(travel_time.duration)
}

/// Doesn't find the closest walking path just the closest based on euclidean distance
fn sort_routes_by_measure<F:Fn(&&crate::stop::Stop)->u64+Clone>(measure: F, routes:&mut [crate::route::Route]){
    routes.sort_unstable_by_key(|route|{
        // we don't need the exactly correct order so making everything an integer key is fine
        let stop = route.stops.iter().min_by_key(measure.clone()).unwrap();

        measure(&stop)
    });

}


//const ARB_DIST:u64 = 1_000_000;
const BU: crate::types::Coords = crate::types::Coords{lat:42.08707019482314, lon:-75.96706048564714};
const BC_BUS_HUB: crate::types::Coords = crate::types::Coords{lat:42.10156942719183, lon:-75.91073683134701};
// return a list indicating the bus routes to take in order
fn calc_routes(routes:&mut [crate::route::Route], from:crate::types::Coords, to: crate::types::Coords)->Vec<crate::route::Route>{
    if routes.len() == 0 {
        return Vec::new()
    }

    let measure_distance_to =       walk_distance_measure(to);
    let measure_distance_from =     walk_distance_measure(from);
    let measure_distance_bu =       walk_distance_measure(BU);
    let measure_distance_bc_hub =   walk_distance_measure(BC_BUS_HUB);
    let route_measure_dist_to = move |route: &crate::route::Route|{
        measure_distance_to(&route.stops.iter().min_by_key(measure_distance_to.clone()).unwrap()) 
    };
    let route_measure_dist_from = move |route: &crate::route::Route|{
        measure_distance_from(&route.stops.iter().min_by_key(measure_distance_from.clone()).unwrap()) 
    };
    let route_measure_dist_bu = move |route: &crate::route::Route|{
        measure_distance_bu(&route.stops.iter().min_by_key(measure_distance_bu.clone()).unwrap()) 
    };
    let route_measure_dist_bc_hub = move |route: &crate::route::Route|{
        measure_distance_bc_hub(&route.stops.iter().min_by_key(measure_distance_bc_hub.clone()).unwrap()) 
    };

    let min_direct_route = routes.iter().min_by_key(|route|route_measure_dist_to(route)+route_measure_dist_from(route)).unwrap();
    let min_to_bu_route = routes.iter().min_by_key(|route|route_measure_dist_from(route)+route_measure_dist_bu(route)).unwrap();
    let min_from_bu_route = routes.iter().min_by_key(|route|route_measure_dist_bu(route)+route_measure_dist_to(route)).unwrap();
    let min_to_bc_hub = routes.iter().min_by_key(|route|route_measure_dist_from(route)+route_measure_dist_bc_hub(route)).unwrap();
    let min_from_bc_hub = routes.iter().min_by_key(|route|route_measure_dist_bc_hub(route)+route_measure_dist_to(route)).unwrap();

    let direct_route_cost = route_measure_dist_to(min_direct_route) + route_measure_dist_from(min_direct_route);
    let bu_route_cost = route_measure_dist_from(min_to_bu_route) + route_measure_dist_to(min_from_bu_route);
    let bc_route_cost = route_measure_dist_from(min_to_bc_hub) + route_measure_dist_from(min_from_bc_hub);
    
    select_min(vec![(direct_route_cost,vec![min_direct_route.clone()]), (bu_route_cost,vec![min_to_bu_route.clone(),min_from_bu_route.clone()]), (bc_route_cost, vec![min_to_bc_hub.clone(), min_from_bc_hub.clone()])])
}

fn walk_distance_measure(pos: crate::types::Coords)->impl Fn(&&crate::stop::Stop)->u64+Clone{
    
    move |stop:&&crate::stop::Stop|{
        (((stop.lat-pos.lat).powi(2) + (stop.lon - pos.lon).powi(2)).powf(0.5)*1_000_000.0) as u64
    }
}

fn select_min(dist_route_pairs:Vec<(u64,Vec<crate::route::Route>)>)->Vec<crate::route::Route>{
    let (_, ret) = dist_route_pairs.into_iter().min_by_key(|(dist,_)|dist.clone()).unwrap();
    ret
}
